package starling.animation {
import starling.events.EventDispatcher;
/**
 *  A DelayedCall allows you to execute a method after a certain time has passed. Since it 
 *  *  implements the IAnimatable interface, it can be added to a juggler. In most cases, you 
 *  *  do not have to use this class directly; the juggler class contains a method to delay
 *  *  calls directly. 
 *  * 
 *  *  <p>DelayedCall dispatches an Event of type 'Event.REMOVE_FROM_JUGGLER' when it is finished,
 *  *  so that the juggler automatically removes it when its no longer needed.</p>
 *  * 
 *  *  @see Juggler
 *  
 * @externs
 */
public class DelayedCall extends starling.events.EventDispatcher implements starling.animation.IAnimatable {
	/**
	 *  Creates a delayed call. 
	 */
	public function DelayedCall(callback:Function, delay:Number, args:Array = undefined) {
		super();
	}
	/**
	 *  Resets the delayed call to its default values, which is useful for pooling. 
	 */
	public function reset(callback:Function, delay:Number, args:Array = undefined):starling.animation.DelayedCall { return null; }
	/**
	 *  @inheritDoc 
	 */
	public function advanceTime(time:Number):void {}
	/**
	 *  Advances the delayed call so that it is executed right away. If 'repeatCount' is
	 *       * anything else than '1', this method will complete only the current iteration. 
	 */
	public function complete():void {}
	/**
	 *  Indicates if enough time has passed, and the call has already been executed. 
	 */
	public function get isComplete():Boolean { return false; }
	/**
	 *  The time for which calls will be delayed (in seconds). 
	 */
	public function get totalTime():Number { return 0; }
	/**
	 *  The time that has already passed (in seconds). 
	 */
	public function get currentTime():Number { return 0; }
	/**
	 *  The number of times the call will be repeated. 
	 *      * Set to '0' to repeat indefinitely. @default 1 
	 */
	public function get repeatCount():int { return 0; }
	public function set repeatCount(value:int):void {}
	/**
	 *  The callback that will be executed when the time is up. 
	 */
	public function get callback():Function { return null; }
	/**
	 *  The arguments that the callback will be executed with.
	 *         *  Beware: not a copy, but the actual object! 
	 */
	public function get arguments():Array { return null; }
}
}
