package starling.display {
/**
 *  A class that provides constant values for visual blend mode effects. 
 *  *   
 *  *  <p>A blend mode is always defined by two 'Context3DBlendFactor' values. A blend factor 
 *  *  represents a particular four-value vector that is multiplied with the source or destination
 *  *  color in the blending formula. The blending formula is:</p>
 *  * 
 *  *  <pre>result = source � sourceFactor + destination � destinationFactor</pre>
 *  * 
 *  *  <p>In the formula, the source color is the output color of the pixel shader program. The 
 *  *  destination color is the color that currently exists in the color buffer, as set by 
 *  *  previous clear and draw operations.</p>
 *  *  
 *  *  <p>Beware that blending factors produce different output depending on the texture type.
 *  *  Textures may contain 'premultiplied alpha' (pma), which means that their RGB values were 
 *  *  multiplied with their alpha value (to save processing time). Textures based on 'BitmapData'
 *  *  objects have premultiplied alpha values, while ATF textures haven't. For this reason, 
 *  *  a blending mode may have different factors depending on the pma value.</p>
 *  *  
 *  *  @see openfl.display3D.Context3DBlendFactor
 *  
 * @externs
 */
public class BlendMode {
	/**
	 *  Creates a new BlendMode instance. Don't call this method directly; instead,
	 *      *  register a new blend mode using <code>BlendMode.register</code>. 
	 */
	public function BlendMode(name:String, sourceFactor:String, destinationFactor:String) {}
	/**
	 *  Inherits the blend mode from this display object's parent. 
	 */
	public static const AUTO:String = "auto";
	/**
	 *  Deactivates blending, i.e. disabling any transparency. 
	 */
	public static const NONE:String = "none";
	/**
	 *  The display object appears in front of the background. 
	 */
	public static const NORMAL:String = "normal";
	/**
	 *  Adds the values of the colors of the display object to the colors of its background. 
	 */
	public static const ADD:String = "add";
	/**
	 *  Multiplies the values of the display object colors with the the background color. 
	 */
	public static const MULTIPLY:String = "multiply";
	/**
	 *  Multiplies the complement (inverse) of the display object color with the complement of 
	 *       * the background color, resulting in a bleaching effect. 
	 */
	public static const SCREEN:String = "screen";
	/**
	 *  Erases the background when drawn on a RenderTexture. 
	 */
	public static const ERASE:String = "erase";
	/**
	 *  When used on a RenderTexture, the drawn object will act as a mask for the current
	 *      * content, i.e. the source alpha overwrites the destination alpha. 
	 */
	public static const MASK:String = "mask";
	/**
	 *  Draws under/below existing objects; useful especially on RenderTextures. 
	 */
	public static const BELOW:String = "below";
	/**
	 *  Returns the blend mode with the given name.
	 *      *  Throws an ArgumentError if the mode does not exist. 
	 */
	public static function get(modeName:String):starling.display.BlendMode { return null; }
	/**
	 *  Returns allready registered blend mode by
	 *     * given blend mode factors. Returns null if not exist.
	 */
	public static function getByFactors(srcFactor:String, dstFactor:String):starling.display.BlendMode { return null; }
	/**
	 *  Registers a blending mode under a certain name. 
	 */
	public static function register(name:String, srcFactor:String, dstFactor:String):starling.display.BlendMode { return null; }
	/**
	 *  Returns an array with all currently registered blend modes. 
	 */
	public static function getAll(out:Array = undefined):Array { return null; }
	/**
	 *  Returns true if a blend mode with the given name is available. 
	 */
	public static function isRegistered(modeName:String):Boolean { return false; }
	/**
	 *  Sets the appropriate blend factors for source and destination on the current context. 
	 */
	public function activate():void {}
	/**
	 *  Returns the name of the blend mode. 
	 */
	public function toString():String { return null; }
	/**
	 *  The source blend factor of this blend mode. 
	 */
	public function get sourceFactor():String { return null; }
	/**
	 *  The destination blend factor of this blend mode. 
	 */
	public function get destinationFactor():String { return null; }
	/**
	 *  Returns the name of the blend mode. 
	 */
	public function get name():String { return null; }
}
}
