package starling.display {
import starling.rendering.Painter;
import starling.filters.FragmentFilter;
import starling.events.EventDispatcher;
import starling.events.Event;
import openfl.geom.Vector3D;
import openfl.geom.Rectangle;
import openfl.geom.Point;
import openfl.geom.Matrix3D;
import openfl.geom.Matrix;
import openfl.display.BitmapData;
/**
 *  Dispatched when an object is added to a parent. 
 * @externs
 */
public class DisplayObject extends starling.events.EventDispatcher {
	/**
	 *  @private 
	 */
	function DisplayObject() {
		super();
	}
	/**
	 *  Disposes all resources of the display object. 
	 *       * GPU buffers are released, event listeners are removed, filters and masks are disposed. 
	 */
	public function dispose():void {}
	/**
	 *  Removes the object from its parent, if it has one, and optionally disposes it. 
	 */
	public function removeFromParent(dispose:Boolean = undefined):void {}
	/**
	 *  Creates a matrix that represents the transformation from the local coordinate system 
	 *      * to another. If you pass an <code>out</code>-matrix, the result will be stored in this matrix
	 *      * instead of creating a new object. 
	 */
	public function getTransformationMatrix(targetSpace:starling.display.DisplayObject, out:openfl.geom.Matrix = undefined):openfl.geom.Matrix { return null; }
	/**
	 *  Returns a rectangle that completely encloses the object as it appears in another 
	 *      * coordinate system. If you pass an <code>out</code>-rectangle, the result will be stored in this 
	 *      * rectangle instead of creating a new object. 
	 */
	public function getBounds(targetSpace:starling.display.DisplayObject, out:openfl.geom.Rectangle = undefined):openfl.geom.Rectangle { return null; }
	/**
	 *  Returns the object that is found topmost beneath a point in local coordinates, or nil
	 *      *  if the test fails. Untouchable and invisible objects will cause the test to fail. 
	 */
	public function hitTest(localPoint:openfl.geom.Point):starling.display.DisplayObject { return null; }
	/**
	 *  Checks if a certain point is inside the display object's mask. If there is no mask,
	 *      * this method always returns <code>true</code> (because having no mask is equivalent
	 *      * to having one that's infinitely big). 
	 */
	public function hitTestMask(localPoint:openfl.geom.Point):Boolean { return false; }
	/**
	 *  Transforms a point from the local coordinate system to global (stage) coordinates.
	 *      * If you pass an <code>out</code>-point, the result will be stored in this point instead of 
	 *      * creating a new object. 
	 */
	public function localToGlobal(localPoint:openfl.geom.Point, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Transforms a point from global (stage) coordinates to the local coordinate system.
	 *      * If you pass an <code>out</code>-point, the result will be stored in this point instead of 
	 *      * creating a new object. 
	 */
	public function globalToLocal(globalPoint:openfl.geom.Point, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Renders the display object with the help of a painter object. Never call this method
	 *      *  directly, except from within another render method.
	 *      *
	 *      *  @param painter Captures the current render state and provides utility functions
	 *      *                 for rendering.
	 *      
	 */
	public function render(painter:starling.rendering.Painter):void {}
	/**
	 *  Moves the pivot point to a certain position within the local coordinate system
	 *      * of the object. If you pass no arguments, it will be centered. 
	 */
	public function alignPivot(horizontalAlign:String = undefined, verticalAlign:String = undefined):void {}
	/**
	 *  Draws the object into a BitmapData object.
	 * 	 *
	 * 	 *  <p>This is achieved by drawing the object into the back buffer and then copying the
	 * 	 *  pixels of the back buffer into a texture. Beware: image sizes bigger than the back
	 * 	 *  buffer are only supported in AIR version 25 or higher and NOT in Flash Player.</p>
	 * 	 *
	 * 	 *  @param out   If you pass null, the object will be created for you.
	 * 	 *               If you pass a BitmapData object, it should have the size of the
	 * 	 *               object bounds, multiplied by the current contentScaleFactor.
	 * 	 *  @param color The RGB color value with which the bitmap will be initialized.
	 * 	 *  @param alpha The alpha value with which the bitmap will be initialized.
	 * 	 
	 */
	public function drawToBitmapData(out:openfl.display.BitmapData = undefined, color:int = undefined, alpha:Number = undefined):openfl.display.BitmapData { return null; }
	/**
	 *  Creates a matrix that represents the transformation from the local coordinate system
	 *      * to another. This method supports three dimensional objects created via 'Sprite3D'.
	 *      * If you pass an <code>out</code>-matrix, the result will be stored in this matrix
	 *      * instead of creating a new object. 
	 */
	public function getTransformationMatrix3D(targetSpace:starling.display.DisplayObject, out:openfl.geom.Matrix3D = undefined):openfl.geom.Matrix3D { return null; }
	/**
	 *  Transforms a 3D point from the local coordinate system to global (stage) coordinates.
	 *      * This is achieved by projecting the 3D point onto the (2D) view plane.
	 *      *
	 *      * <p>If you pass an <code>out</code>-point, the result will be stored in this point instead of
	 *      * creating a new object.</p> 
	 */
	public function local3DToGlobal(localPoint:openfl.geom.Vector3D, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Transforms a point from global (stage) coordinates to the 3D local coordinate system.
	 *      * If you pass an <code>out</code>-vector, the result will be stored in this point instead of
	 *      * creating a new object. 
	 */
	public function globalToLocal3D(globalPoint:openfl.geom.Point, out:openfl.geom.Vector3D = undefined):openfl.geom.Vector3D { return null; }
	/**
	 *  Forces the object to be redrawn in the next frame.
	 *      *  This will prevent the object to be drawn from the render cache.
	 *      *
	 *      *  <p>This method is called every time the object changes in any way. When creating
	 *      *  custom mesh styles or any other custom rendering code, call this method if the object
	 *      *  needs to be redrawn.</p>
	 *      *
	 *      *  <p>If the object needs to be redrawn just because it does not support the render cache,
	 *      *  call <code>painter.excludeFromCache()</code> in the object's render method instead.
	 *      *  That way, Starling's <code>skipUnchangedFrames</code> policy won't be disrupted.</p>
	 *      
	 */
	public function setRequiresRedraw():void {}
	/**
	 *  Indicates if the object needs to be redrawn in the upcoming frame, i.e. if it has
	 *      *  changed its location relative to the stage or some other aspect of its appearance
	 *      *  since it was last rendered. 
	 */
	public function get requiresRedraw():Boolean { return false; }
	/**
	 *  @private 
	 */
	override public function dispatchEvent(event:starling.events.Event):void {}
	/**
	 *  @inheritDoc 
	 */
	override public function addEventListener(type:String, listener:Function):void {}
	/**
	 *  @inheritDoc 
	 */
	override public function removeEventListener(type:String, listener:Function):void {}
	/**
	 *  @inheritDoc 
	 */
	override public function removeEventListeners(type:String = undefined):void {}
	/**
	 *  The transformation matrix of the object relative to its parent.
	 *      * 
	 *      * <p>If you assign a custom transformation matrix, Starling will try to figure out  
	 *      * suitable values for <code>x, y, scaleX, scaleY,</code> and <code>rotation</code>.
	 *      * However, if the matrix was created in a different way, this might not be possible. 
	 *      * In that case, Starling will apply the matrix, but not update the corresponding 
	 *      * properties.</p>
	 *      * 
	 *      * <p>CAUTION: not a copy, but the actual object!</p> 
	 */
	public function get transformationMatrix():openfl.geom.Matrix { return null; }
	public function set transformationMatrix(value:openfl.geom.Matrix):void {}
	/**
	 *  The 3D transformation matrix of the object relative to its parent.
	 *      *
	 *      * <p>For 2D objects, this property returns just a 3D version of the 2D transformation
	 *      * matrix. Only the 'Sprite3D' class supports real 3D transformations.</p>
	 *      *
	 *      * <p>CAUTION: not a copy, but the actual object!</p> 
	 */
	public function get transformationMatrix3D():openfl.geom.Matrix3D { return null; }
	/**
	 *  Indicates if this object or any of its parents is a 'Sprite3D' object. 
	 */
	public function get is3D():Boolean { return false; }
	/**
	 *  Indicates if the mouse cursor should transform into a hand while it's over the sprite. 
	 *      * @default false 
	 */
	public function get useHandCursor():Boolean { return false; }
	public function set useHandCursor(value:Boolean):void {}
	/**
	 *  The bounds of the object relative to the local coordinates of the parent. 
	 */
	public function get bounds():openfl.geom.Rectangle { return null; }
	/**
	 *  The width of the object in pixels.
	 *      * Note that for objects in a 3D space (connected to a Sprite3D), this value might not
	 *      * be accurate until the object is part of the display list. 
	 */
	public function get width():Number { return 0; }
	public function set width(value:Number):void {}
	/**
	 *  The height of the object in pixels.
	 *      * Note that for objects in a 3D space (connected to a Sprite3D), this value might not
	 *      * be accurate until the object is part of the display list. 
	 */
	public function get height():Number { return 0; }
	public function set height(value:Number):void {}
	/**
	 *  The x coordinate of the object relative to the local coordinates of the parent. 
	 */
	public function get x():Number { return 0; }
	public function set x(value:Number):void {}
	/**
	 *  The y coordinate of the object relative to the local coordinates of the parent. 
	 */
	public function get y():Number { return 0; }
	public function set y(value:Number):void {}
	/**
	 *  The x coordinate of the object's origin in its own coordinate space (default: 0). 
	 */
	public function get pivotX():Number { return 0; }
	public function set pivotX(value:Number):void {}
	/**
	 *  The y coordinate of the object's origin in its own coordinate space (default: 0). 
	 */
	public function get pivotY():Number { return 0; }
	public function set pivotY(value:Number):void {}
	/**
	 *  The horizontal scale factor. '1' means no scale, negative values flip the object.
	 *      * @default 1 
	 */
	public function get scaleX():Number { return 0; }
	public function set scaleX(value:Number):void {}
	/**
	 *  The vertical scale factor. '1' means no scale, negative values flip the object.
	 *      * @default 1 
	 */
	public function get scaleY():Number { return 0; }
	public function set scaleY(value:Number):void {}
	/**
	 *  Sets both 'scaleX' and 'scaleY' to the same value. The getter simply returns the
	 *      * value of 'scaleX' (even if the scaling values are different). @default 1 
	 */
	public function get scale():Number { return 0; }
	public function set scale(value:Number):void {}
	/**
	 *  The horizontal skew angle in radians. 
	 */
	public function get skewX():Number { return 0; }
	public function set skewX(value:Number):void {}
	/**
	 *  The vertical skew angle in radians. 
	 */
	public function get skewY():Number { return 0; }
	public function set skewY(value:Number):void {}
	/**
	 *  The rotation of the object in radians. (In Starling, all angles are measured 
	 *      * in radians.) 
	 */
	public function get rotation():Number { return 0; }
	public function set rotation(value:Number):void {}
	/**
	 *  The opacity of the object. 0 = transparent, 1 = opaque. @default 1 
	 */
	public function get alpha():Number { return 0; }
	public function set alpha(value:Number):void {}
	/**
	 *  The visibility of the object. An invisible object will be untouchable. 
	 */
	public function get visible():Boolean { return false; }
	public function set visible(value:Boolean):void {}
	/**
	 *  Indicates if this object (and its children) will receive touch events. 
	 */
	public function get touchable():Boolean { return false; }
	public function set touchable(value:Boolean):void {}
	/**
	 *  The blend mode determines how the object is blended with the objects underneath. 
	 *      * @default auto
	 *      * @see starling.display.BlendMode 
	 */
	public function get blendMode():String { return null; }
	public function set blendMode(value:String):void {}
	/**
	 *  The name of the display object (default: null). Used by 'getChildByName()' of 
	 *      * display object containers. 
	 */
	public function get name():String { return null; }
	public function set name(value:String):void {}
	/**
	 *  The filter that is attached to the display object. The <code>starling.filters</code>
	 *      *  package contains several classes that define specific filters you can use. To combine
	 *      *  several filters, assign an instance of the <code>FilterChain</code> class; to remove
	 *      *  all filters, assign <code>null</code>.
	 *      *
	 *      *  <p>Beware that a filter instance may only be used on one object at a time! Furthermore,
	 *      *  when you remove or replace a filter, it is NOT disposed automatically (since you might
	 *      *  want to reuse it on a different object).</p>
	 *      *
	 *      *  @default null
	 *      *  @see starling.filters.FragmentFilter
	 *      *  @see starling.filters.FilterChain
	 *      
	 */
	public function get filter():starling.filters.FragmentFilter { return null; }
	public function set filter(value:starling.filters.FragmentFilter):void {}
	/**
	 *  The display object that acts as a mask for the current object.
	 *      *  Assign <code>null</code> to remove it.
	 *      *
	 *      *  <p>A pixel of the masked display object will only be drawn if it is within one of the
	 *      *  mask's polygons. Texture pixels and alpha values of the mask are not taken into
	 *      *  account. The mask object itself is never visible.</p>
	 *      *
	 *      *  <p>If the mask is part of the display list, masking will occur at exactly the
	 *      *  location it occupies on the stage. If it is not, the mask will be placed in the local
	 *      *  coordinate system of the target object (as if it was one of its children).</p>
	 *      *
	 *      *  <p>For rectangular masks, you can use simple quads; for other forms (like circles
	 *      *  or arbitrary shapes) it is recommended to use a 'Canvas' instance.</p>
	 *      *
	 *      *  <p><strong>Note:</strong> a mask will typically cause at least two additional draw
	 *      *  calls: one to draw the mask to the stencil buffer and one to erase it. However, if the
	 *      *  mask object is an instance of <code>starling.display.Quad</code> and is aligned
	 *      *  parallel to the stage axes, rendering will be optimized: instead of using the
	 *      *  stencil buffer, the object will be clipped using the scissor rectangle. That's
	 *      *  faster and reduces the number of draw calls, so make use of this when possible.</p>
	 *      *
	 *      *  <p><strong>Note:</strong> AIR apps require the <code>depthAndStencil</code> node
	 *      *  in the application descriptor XMLs to be enabled! Otherwise, stencil masking won't
	 *      *  work.</p>
	 *      *
	 *      *  @see Canvas
	 *      *  @default null
	 *      
	 */
	public function get mask():starling.display.DisplayObject { return null; }
	public function set mask(value:starling.display.DisplayObject):void {}
	/**
	 *  Indicates if the masked region of this object is set to be inverted.
	 */
	public function get maskInverted():Boolean { return false; }
	public function set maskInverted(value:Boolean):void {}
	/**
	 *  The display object container that contains this display object. 
	 */
	public function get parent():starling.display.DisplayObjectContainer { return null; }
	/**
	 *  The topmost object in the display tree the object is part of. 
	 */
	public function get base():starling.display.DisplayObject { return null; }
	/**
	 *  The root object the display object is connected to (i.e. an instance of the class 
	 *      * that was passed to the Starling constructor), or null if the object is not connected
	 *      * to the stage. 
	 */
	public function get root():starling.display.DisplayObject { return null; }
	/**
	 *  The stage the display object is connected to, or null if it is not connected 
	 *      * to the stage. 
	 */
	public function get stage():starling.display.Stage { return null; }
}
}
