package starling.events {
import starling.display.DisplayObject;
import openfl.geom.Point;
/**
 *  A Touch object contains information about the presence or movement of a finger
 *  *  or the mouse on the screen.
 *  *  
 *  *  <p>You receive objects of this type from a TouchEvent. When such an event is triggered,
 *  *  you can query it for all touches that are currently present on the screen. One touch
 *  *  object contains information about a single touch; it always transitions through a series
 *  *  of TouchPhases. Have a look at the TouchPhase class for more information.</p>
 *  *  
 *  *  <strong>The position of a touch</strong>
 *  *  
 *  *  <p>You can get the current and previous position in stage coordinates with the corresponding 
 *  *  properties. However, you'll want to have the position in a different coordinate system 
 *  *  most of the time. For this reason, there are methods that convert the current and previous 
 *  *  touches into the local coordinate system of any object.</p>
 *  * 
 *  *  @see TouchEvent
 *  *  @see TouchPhase
 *  
 * @externs
 */
public class Touch {
	/**
	 *  Creates a new Touch object. 
	 */
	public function Touch(id:int) {}
	/**
	 *  Converts the current location of a touch to the local coordinate system of a display 
	 *      * object. If you pass an <code>out</code>-point, the result will be stored in this point
	 *      * instead of creating a new object.
	 */
	public function getLocation(space:starling.display.DisplayObject, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Converts the previous location of a touch to the local coordinate system of a display 
	 *      * object. If you pass an <code>out</code>-point, the result will be stored in this point
	 *      * instead of creating a new object.
	 */
	public function getPreviousLocation(space:starling.display.DisplayObject, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Converts the start location of a touch (i.e. when it entered 'TouchPhase.BEGAN') to
	 *      *  the local coordinate system of a display object. If you pass an <code>out</code>-point,
	 *      *  the result will be stored in this point instead of creating a new object.
	 */
	public function getStartLocation(space:starling.display.DisplayObject, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Returns the movement of the touch between the current and previous location. 
	 *      * If you pass an <code>out</code>-point, the result will be stored in this point instead
	 *      * of creating a new object. 
	 */
	public function getMovement(space:starling.display.DisplayObject, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Returns the movement of the touch between the current and the start location.
	 *         *  If you pass an <code>out</code>-point, the result will be stored in this point instead
	 *         *  of creating a new object. 
	 */
	public function getMovementSinceStart(space:starling.display.DisplayObject, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Indicates if the target or one of its children is touched. 
	 */
	public function isTouching(target:starling.display.DisplayObject):Boolean { return false; }
	/**
	 *  Returns a description of the object. 
	 */
	public function toString():String { return null; }
	/**
	 *  Creates a clone of the Touch object. 
	 */
	public function clone():starling.events.Touch { return null; }
	/**
	 *  The identifier of a touch. '0' for mouse events, an increasing number for touches. 
	 */
	public function get id():int { return 0; }
	/**
	 *  The previous x-position of the touch in stage coordinates. 
	 */
	public function get previousGlobalX():Number { return 0; }
	/**
	 *  The previous y-position of the touch in stage coordinates. 
	 */
	public function get previousGlobalY():Number { return 0; }
	/**
	 *  The x-position of the touch when it was in 'TouchPhase.BEGAN', in stage coordinates.
	 *      *  (Or, if the touch is in phase 'HOVER', the x-position when it first appeared.) 
	 */
	public function get startGlobalX():Number { return 0; }
	/**
	 *  The y-position of the touch when it was in 'TouchPhase.BEGAN', in stage coordinates.
	 *      *  (Or, if the touch is in phase 'HOVER', the y-position when it first appeared.) 
	 */
	public function get startGlobalY():Number { return 0; }
	/**
	 *  The x-position of the touch in stage coordinates. If you change this value,
	 *      * the previous one will be moved to "previousGlobalX". 
	 */
	public function get globalX():Number { return 0; }
	public function set globalX(value:Number):void {}
	/**
	 *  The y-position of the touch in stage coordinates. If you change this value,
	 *      * the previous one will be moved to "previousGlobalY". 
	 */
	public function get globalY():Number { return 0; }
	public function set globalY(value:Number):void {}
	/**
	 *  The number of taps the finger made in a short amount of time. Use this to detect 
	 *      * double-taps / double-clicks, etc. 
	 */
	public function get tapCount():int { return 0; }
	public function set tapCount(value:int):void {}
	/**
	 *  The current phase the touch is in. @see TouchPhase 
	 */
	public function get phase():String { return null; }
	public function set phase(value:String):void {}
	/**
	 *  The display object at which the touch occurred. 
	 */
	public function get target():starling.display.DisplayObject { return null; }
	public function set target(value:starling.display.DisplayObject):void {}
	/**
	 *  The moment the touch occurred (in seconds since application start). 
	 */
	public function get timestamp():Number { return 0; }
	public function set timestamp(value:Number):void {}
	/**
	 *  A value between 0.0 and 1.0 indicating force of the contact with the device. 
	 *      * If the device does not support detecting the pressure, the value is 1.0. 
	 */
	public function get pressure():Number { return 0; }
	public function set pressure(value:Number):void {}
	/**
	 *  Width of the contact area. 
	 *      * If the device does not support detecting the pressure, the value is 1.0. 
	 */
	public function get width():Number { return 0; }
	public function set width(value:Number):void {}
	/**
	 *  Height of the contact area. 
	 *      * If the device does not support detecting the pressure, the value is 1.0. 
	 */
	public function get height():Number { return 0; }
	public function set height(value:Number):void {}
	/**
	 *  Indicates if the touch has been cancelled, which may happen when the app moves into
	 *      * the background ('Event.DEACTIVATE'). @default false 
	 */
	public function get cancelled():Boolean { return false; }
	public function set cancelled(value:Boolean):void {}
	/**
	 *  If the touch is hovering, returns the time (in seconds) since the touch was created;
	 *      *  afterwards, the time since the touch was in phase 'BEGAN'. 
	 */
	public function get duration():Number { return 0; }
}
}
