package starling.filters {
import openfl.Vector;
/**
 *  The ColorMatrixFilter class lets you apply a 4x5 matrix transformation to the color
 *  *  and alpha values of every pixel in the input image to produce a result with a new set
 *  *  of color and alpha values. This allows saturation changes, hue rotation,
 *  *  luminance to alpha, and various other effects.
 *  *
 *  *  <p>The class contains several convenience methods for frequently used color
 *  *  adjustments. All those methods change the current matrix, which means you can easily
 *  *  combine them in one filter:</p>
 *  *
 *  *  <listing>
 *  *  // create an inverted filter with 50% saturation and 180° hue rotation
 *  *  var filter:ColorMatrixFilter = new ColorMatrixFilter();
 *  *  filter.invert();
 *  *  filter.adjustSaturation(-0.5);
 *  *  filter.adjustHue(1.0);</listing>
 *  *
 *  *  <p>If you want to gradually animate one of the predefined color adjustments, either reset
 *  *  the matrix after each step, or use an identical adjustment value for each step; the
 *  *  changes will add up.</p>
 *  
 * @externs
 */
public class ColorMatrixFilter extends starling.filters.FragmentFilter {
	/**
	 *  Creates a new ColorMatrixFilter instance with the specified matrix.
	 *      *  @param matrix a vector of 20 items arranged as a 4x5 matrix.
	 *      
	 */
	public function ColorMatrixFilter(matrix:openfl.Vector = undefined) {
		super();
	}
	/**
	 *  Inverts the colors of the filtered object. 
	 */
	public function invert():void {}
	/**
	 *  Changes the saturation. Typical values are in the range (-1, 1).
	 *      *  Values above zero will raise, values below zero will reduce the saturation.
	 *      *  '-1' will produce a grayscale image. 
	 */
	public function adjustSaturation(sat:Number):void {}
	/**
	 *  Changes the contrast. Typical values are in the range (-1, 1).
	 *      *  Values above zero will raise, values below zero will reduce the contrast. 
	 */
	public function adjustContrast(value:Number):void {}
	/**
	 *  Changes the brightness. Typical values are in the range (-1, 1).
	 *      *  Values above zero will make the image brighter, values below zero will make it darker.
	 */
	public function adjustBrightness(value:Number):void {}
	/**
	 *  Changes the hue of the image. Typical values are in the range (-1, 1). 
	 */
	public function adjustHue(value:Number):void {}
	/**
	 *  Tints the image in a certain color, analog to what can be done in Adobe Animate.
	 *      *
	 *      *  @param color   the RGB color with which the image should be tinted.
	 *      *  @param amount  the intensity with which tinting should be applied. Range (0, 1).
	 *      
	 */
	public function tint(color:int, amount:Number = undefined):void {}
	/**
	 *  Changes the filter matrix back to the identity matrix. 
	 */
	public function reset():void {}
	/**
	 *  Concatenates the current matrix with another one. 
	 */
	public function concat(matrix:openfl.Vector):void {}
	/**
	 *  Concatenates the current matrix with another one, passing its contents directly. 
	 */
	public function concatValues(m0:Number, m1:Number, m2:Number, m3:Number, m4:Number, m5:Number, m6:Number, m7:Number, m8:Number, m9:Number, m10:Number, m11:Number, m12:Number, m13:Number, m14:Number, m15:Number, m16:Number, m17:Number, m18:Number, m19:Number):void {}
	/**
	 *  A vector of 20 items arranged as a 4x5 matrix. 
	 */
	public function get matrix():openfl.Vector { return null; }
	public function set matrix(value:openfl.Vector):void {}
	public function get colorEffect():starling.filters.ColorMatrixEffect { return null; }
}
}
