package starling.filters {
import starling.textures.Texture;
import starling.rendering.Painter;
import openfl.geom.Point;
/**
 *  The CompositeFilter class allows to combine several layers of textures into one texture.
 *  *  It's mainly used as a building block for more complex filters; e.g. the DropShadowFilter
 *  *  uses this class to draw the shadow (the result of a BlurFilter) behind an object.
 *  
 * @externs
 */
public class CompositeFilter extends starling.filters.FragmentFilter {
	/**
	 *  Creates a new instance. 
	 */
	public function CompositeFilter() {
		super();
	}
	/**
	 *  Combines up to four input textures into one new texture,
	 *      *  adhering to the properties of each layer. 
	 */
	override public function process(painter:starling.rendering.Painter, helper:starling.filters.IFilterHelper, input0:starling.textures.Texture = undefined, input1:starling.textures.Texture = undefined, input2:starling.textures.Texture = undefined, input3:starling.textures.Texture = undefined):starling.textures.Texture { return null; }
	/**
	 *  Returns the position (in points) at which a certain layer will be drawn. 
	 */
	public function getOffsetAt(layerID:int, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Indicates the position (in points) at which a certain layer will be drawn. 
	 */
	public function setOffsetAt(layerID:int, x:Number, y:Number):void {}
	/**
	 *  Returns the RGB color with which a layer is tinted when it is being drawn.
	 *      *  @default 0xffffff 
	 */
	public function getColorAt(layerID:int):int { return 0; }
	/**
	 *  Indicates if the color of the given layer is replaced (true) or tinted (false). 
	 */
	public function getReplaceColorAt(layerID:int):Boolean { return false; }
	/**
	 *  Adjusts the RGB color with which a layer is tinted when it is being drawn.
	 *      *  If <code>replace</code> is enabled, the pixels are not tinted, but instead
	 *      *  the RGB channels will replace the texture's color entirely.
	 *      
	 */
	public function setColorAt(layerID:int, color:int, replace:Boolean = undefined):void {}
	/**
	 *  Indicates the alpha value with which the layer is drawn.
	 *      *  @default 1.0 
	 */
	public function getAlphaAt(layerID:int):Number { return 0; }
	/**
	 *  Adjusts the alpha value with which the layer is drawn. 
	 */
	public function setAlphaAt(layerID:int, alpha:Number):void {}
	/**
	 *  The mode with which the layer is drawn. @see starling.filters.CompositeMode 
	 */
	public function getModeAt(layerID:int):String { return null; }
	/**
	 *  Sets the mode with which the layer is drawn. @see starling.filters.CompositeMode 
	 */
	public function setModeAt(layerID:int, mode:String):void {}
	/**
	 *  Indicates if the alpha value of the given layer's fragments should be inverted,
	 * 	 *  effectively inverting the layer's silhouette. 
	 */
	public function getInvertAlphaAt(layerID:int):Boolean { return false; }
	/**
	 *  Activate this setting to invert the alpha values of this layer's fragments, which will
	 * 	 *  effectively invert the layer's silhouette. Note that this setting is only applied if
	 * 	 *  color replacement is activated on this layer. 
	 */
	public function setInvertAlphaAt(layerID:int, value:Boolean = undefined):void {}
	public function get compositeEffect():starling.filters.CompositeEffect { return null; }
}
}
