package starling.filters {
import starling.textures.Texture;
import starling.rendering.Painter;
/**
 *  The DropShadowFilter class lets you add a drop shadow to display objects.
 *  *  To create the shadow, the class internally uses the BlurFilter.
 *  
 * @externs
 */
public class DropShadowFilter extends starling.filters.FragmentFilter {
	/**
	 *  Creates a new DropShadowFilter instance with the specified parameters.
	 *      *
	 *      * @param distance   the offset distance of the shadow, in points.
	 *      * @param angle      the angle with which the shadow is offset, in radians.
	 *      * @param color      the color of the shadow.
	 *      * @param alpha      the alpha value of the shadow. Values between 0 and 1 modify the
	 *      *                   opacity; values > 1 will make it stronger, i.e. produce a harder edge.
	 *      * @param blur       the amount of blur with which the shadow is created. Note that high
	 *      *                   values will cause the number of render passes to grow.
	 *      * @param quality 	 the quality of the shadow blur, '1' being the best (range 0.1 - 1.0)
	 * 	 * @param inner      if enabled, the shadow will be drawn inside the object.
	 * 	 * @param knockout   if enabled, only the shadow will be drawn.
	 *      
	 */
	public function DropShadowFilter(distance:Number = undefined, angle:Number = undefined, color:int = undefined, alpha:Number = undefined, blur:Number = undefined, quality:Number = undefined, inner:Boolean = undefined, knockout:Boolean = undefined) {
		super();
	}
	/**
	 *  @inheritDoc 
	 */
	override public function dispose():void {}
	/**
	 *  @private 
	 */
	override public function process(painter:starling.rendering.Painter, helper:starling.filters.IFilterHelper, input0:starling.textures.Texture = undefined, input1:starling.textures.Texture = undefined, input2:starling.textures.Texture = undefined, input3:starling.textures.Texture = undefined):starling.textures.Texture { return null; }
	/**
	 *  The color of the shadow. @default 0x0 
	 */
	public function get color():int { return 0; }
	public function set color(value:int):void {}
	/**
	 *  The alpha value of the shadow. Values between 0 and 1 modify the opacity;
	 *      *  values > 1 will make it stronger, i.e. produce a harder edge. @default 0.5 
	 */
	public function get alpha():Number { return 0; }
	public function set alpha(value:Number):void {}
	/**
	 *  The offset distance for the shadow, in points. @default 4.0 
	 */
	public function get distance():Number { return 0; }
	public function set distance(value:Number):void {}
	/**
	 *  The angle with which the shadow is offset, in radians. @default Math.PI / 4 
	 */
	public function get angle():Number { return 0; }
	public function set angle(value:Number):void {}
	/**
	 *  The amount of blur with which the shadow is created.
	 *      *  The number of required passes will be <code>Math.ceil(value) × 2</code>.
	 *      *  @default 1.0 
	 */
	public function get blur():Number { return 0; }
	public function set blur(value:Number):void {}
	/**
	 *  The quality used for blurring the shadow (range: 0.1 - 1).
	 *      *  Forwarded to the internally used <em>BlurFilter</em>. 
	 */
	public function get quality():Number { return 0; }
	public function set quality(value:Number):void {}
	/**
	 *  Indicates whether or not the shadow is an inner shadow. The default is
	 * 	 *  <code>false</code>, an outer shadow (a shadow around the outer edges of the object). 
	 */
	public function get inner():Boolean { return false; }
	public function set inner(value:Boolean):void {}
	public function get_inner():Boolean { return false; }
	public function set_inner(value:Boolean):Boolean { return false; }
	/**
	 *  If enabled, applies a knockout effect, which effectively makes the object's fill
	 * 	 *  transparent. @default false 
	 */
	public function get knockout():Boolean { return false; }
	public function set knockout(value:Boolean):void {}
	public function get_knockout():Boolean { return false; }
	public function set_knockout(value:Boolean):Boolean { return false; }
}
}
