package starling.geom {
import starling.rendering.VertexData;
import starling.rendering.IndexData;
import openfl.geom.Point;
/**
 *  A polygon describes a closed two-dimensional shape bounded by a number of straight
 *  *  line segments.
 *  *
 *  *  <p>The vertices of a polygon form a closed path (i.e. the last vertex will be connected
 *  *  to the first). It is recommended to provide the vertices in clockwise order.
 *  *  Self-intersecting paths are not supported and will give wrong results on triangulation,
 *  *  area calculation, etc.</p>
 *  
 * @externs
 */
public class Polygon {
	/**
	 *  Creates a Polygon with the given coordinates.
	 *      * @param vertices an array that contains either 'Point' instances or
	 *      *                 alternating 'x' and 'y' coordinates.
	 *      
	 */
	public function Polygon(vertices:Array = undefined) {}
	/**
	 *  Creates an ellipse with optimized implementations of triangulation, hitTest, etc. 
	 */
	public static function createEllipse(x:Number, y:Number, radiusX:Number, radiusY:Number, numSides:int = undefined):starling.geom.Polygon { return null; }
	/**
	 *  Creates a circle with optimized implementations of triangulation, hitTest, etc. 
	 */
	public static function createCircle(x:Number, y:Number, radius:Number, numSides:int = undefined):starling.geom.Polygon { return null; }
	/**
	 *  Creates a rectangle with optimized implementations of triangulation, hitTest, etc. 
	 */
	public static function createRectangle(x:Number, y:Number, width:Number, height:Number):starling.geom.Polygon { return null; }
	/**
	 *  Creates a clone of this polygon. 
	 */
	public function clone():starling.geom.Polygon { return null; }
	/**
	 *  Reverses the order of the vertices. Note that some methods of the Polygon class
	 *      * require the vertices in clockwise order. 
	 */
	public function reverse():void {}
	/**
	 *  Adds vertices to the polygon. Pass either a list of 'Point' instances or alternating
	 *      * 'x' and 'y' coordinates. 
	 */
	public function addVertices(args:Array):void {}
	/**
	 *  Moves a given vertex to a certain position or adds a new vertex at the end. 
	 */
	public function setVertex(index:int, x:Number, y:Number):void {}
	/**
	 *  Returns the coordinates of a certain vertex. 
	 */
	public function getVertex(index:int, out:openfl.geom.Point = undefined):openfl.geom.Point { return null; }
	/**
	 *  Figures out if the given coordinates lie within the polygon. 
	 */
	public function contains(x:Number, y:Number):Boolean { return false; }
	/**
	 *  Figures out if the given point lies within the polygon. 
	 */
	public function containsPoint(point:openfl.geom.Point):Boolean { return false; }
	/**
	 *  Calculates a possible representation of the polygon via triangles. The resulting
	 *      *  IndexData instance will reference the polygon vertices as they are saved in this
	 *      *  Polygon instance, optionally incremented by the given offset.
	 *      *
	 *      *  <p>If you pass an indexData object, the new indices will be appended to it.
	 *      *  Otherwise, a new instance will be created.</p> 
	 */
	public function triangulate(indexData:starling.rendering.IndexData = undefined, offset:int = undefined):starling.rendering.IndexData { return null; }
	/**
	 *  Copies all vertices to a 'VertexData' instance, beginning at a certain target index. 
	 */
	public function copyToVertexData(target:starling.rendering.VertexData, targetVertexID:int = undefined, attrName:String = undefined):void {}
	/**
	 *  Creates a string that contains the values of all included points. 
	 */
	public function toString():String { return null; }
	/**
	 *  Indicates if the polygon's line segments are not self-intersecting.
	 *      * Beware: this is a brute-force implementation with <code>O(n^2)</code>. 
	 */
	public function get isSimple():Boolean { return false; }
	/**
	 *  Indicates if the polygon is convex. In a convex polygon, the vector between any two
	 *      * points inside the polygon lies inside it, as well. 
	 */
	public function get isConvex():Boolean { return false; }
	/**
	 *  Calculates the total area of the polygon. 
	 */
	public function get area():Number { return 0; }
	/**
	 *  Returns the total number of vertices spawning up the polygon. Assigning a value
	 *      * that's smaller than the current number of vertices will crop the path; a bigger
	 *      * value will fill up the path with zeros. 
	 */
	public function get numVertices():int { return 0; }
	public function set numVertices(value:int):void {}
	/**
	 *  Returns the number of triangles that will be required when triangulating the polygon. 
	 */
	public function get numTriangles():int { return 0; }
}
}
