package starling.rendering {
import starling.utils.MeshSubset;
import starling.display.MeshBatch;
import starling.display.Mesh;
/**
 *  This class manages a list of mesh batches of different types;
 *  *  it acts as a "meta" MeshBatch that initiates all rendering.
 *  
 * @externs
 */
public class BatchProcessor {
	/**
	 *  Creates a new batch processor. 
	 */
	public function BatchProcessor() {}
	/**
	 *  Disposes all batches (including those in the reusable pool). 
	 */
	public function dispose():void {}
	/**
	 *  Adds a mesh to the current batch, or to a new one if the current one does not support
	 *      *  it. Whenever the batch changes, <code>onBatchComplete</code> is called for the previous
	 *      *  one.
	 *      *
	 *      *  @param mesh       the mesh to add to the current (or new) batch.
	 *      *  @param state      the render state from which to take the current settings for alpha,
	 *      *                    modelview matrix, and blend mode.
	 *      *  @param subset     the subset of the mesh you want to add, or <code>null</code> for
	 *      *                    the complete mesh.
	 *      *  @param ignoreTransformations   when enabled, the mesh's vertices will be added
	 *      *                    without transforming them in any way (no matter the value of the
	 *      *                    state's <code>modelviewMatrix</code>).
	 *      
	 */
	public function addMesh(mesh:starling.display.Mesh, state:starling.rendering.RenderState, subset:starling.utils.MeshSubset = undefined, ignoreTransformations:Boolean = undefined):void {}
	/**
	 *  Finishes the current batch, i.e. call the 'onComplete' callback on the batch and
	 *      *  prepares initialization of a new one. 
	 */
	public function finishBatch():void {}
	/**
	 *  Clears all batches and adds them to a pool so they can be reused later. 
	 */
	public function clear():void {}
	/**
	 *  Returns the batch at a certain index. 
	 */
	public function getBatchAt(batchID:int):starling.display.MeshBatch { return null; }
	/**
	 *  Disposes all batches that are currently unused. 
	 */
	public function trim():void {}
	/**
	 *  Sets all properties of the given token so that it describes the current position
	 *      *  within this instance. 
	 */
	public function fillToken(token:starling.rendering.BatchToken):starling.rendering.BatchToken { return null; }
	/**
	 *  The number of batches currently stored in the BatchProcessor. 
	 */
	public function get numBatches():int { return 0; }
	/**
	 *  This callback is executed whenever a batch is finished and replaced by a new one.
	 *      *  The finished MeshBatch is passed to the callback. Typically, this callback is used
	 *      *  to actually render it. 
	 */
	public function get onBatchComplete():Function { return null; }
	public function set onBatchComplete(value:Function):void {}
}
}
