package starling.text {
import starling.textures.Texture;
import starling.styles.MeshStyle;
import starling.display.Sprite;
import starling.display.MeshBatch;
import openfl.Vector;
/**
 *  The BitmapFont class parses bitmap font files and arranges the glyphs
 *  *  in the form of a text.
 *  *
 *  *  The class parses the XML format as it is used in the 
 *  *  <a href="http://www.angelcode.com/products/bmfont/">AngelCode Bitmap Font Generator</a> or
 *  *  the <a href="http://glyphdesigner.71squared.com/">Glyph Designer</a>. 
 *  *  This is what the file format looks like:
 *  *
 *  *  <pre> 
 *  *  &lt;font&gt;
 *  *    &lt;info face="BranchingMouse" size="40" /&gt;
 *  *    &lt;common lineHeight="40" /&gt;
 *  *    &lt;pages&gt;  &lt;!-- currently, only one page is supported --&gt;
 *  *      &lt;page id="0" file="texture.png" /&gt;
 *  *    &lt;/pages&gt;
 *  *    &lt;chars&gt;
 *  *      &lt;char id="32" x="60" y="29" width="1" height="1" xoffset="0" yoffset="27" xadvance="8" /&gt;
 *  *      &lt;char id="33" x="155" y="144" width="9" height="21" xoffset="0" yoffset="6" xadvance="9" /&gt;
 *  *    &lt;/chars&gt;
 *  *    &lt;kernings&gt; &lt;!-- Kerning is optional --&gt;
 *  *      &lt;kerning first="83" second="83" amount="-4"/&gt;
 *  *    &lt;/kernings&gt;
 *  *  &lt;/font&gt;
 *  *  </pre>
 *  *  
 *  *  Pass an instance of this class to the method <code>registerBitmapFont</code> of the
 *  *  TextField class. Then, set the <code>fontName</code> property of the text field to the 
 *  *  <code>name</code> value of the bitmap font. This will make the text field use the bitmap
 *  *  font.  
 *  
 * @externs
 */
public class BitmapFont implements starling.text.ITextCompositor {
	/**
	 *  Creates a bitmap font from the given texture and font data.
	 *      *  If you don't pass any data, the "mini" font will be created.
	 *      *
	 *      * @param texture  The texture containing all the glyphs.
	 *      * @param fontData Typically an XML file in the standard AngelCode format. Override the
	 *      *                 the 'parseFontData' method to add support for additional formats.
	 *      
	 */
	public function BitmapFont(texture:starling.textures.Texture = undefined, fontData:* = undefined) {}
	/**
	 *  Use this constant for the <code>fontSize</code> property of the TextField class to 
	 *      * render the bitmap font in exactly the size it was created. 
	 */
	public static const NATIVE_SIZE:int = -1;
	/**
	 *  The font name of the embedded minimal bitmap font. Use this e.g. for debug output. 
	 */
	public static const MINI:String = "mini";
	/**
	 *  Disposes the texture of the bitmap font! 
	 */
	public function dispose():void {}
	/**
	 *  Returns a single bitmap char with a certain character ID. 
	 */
	public function getChar(charID:int):starling.text.BitmapChar { return null; }
	/**
	 *  Adds a bitmap char with a certain character ID. 
	 */
	public function addChar(charID:int, bitmapChar:starling.text.BitmapChar):void {}
	/**
	 *  Returns a vector containing all the character IDs that are contained in this font. 
	 */
	public function getCharIDs(result:openfl.Vector = undefined):openfl.Vector { return null; }
	/**
	 *  Checks whether a provided string can be displayed with the font. 
	 */
	public function hasChars(text:String):Boolean { return false; }
	/**
	 *  Creates a sprite that contains a certain text, made up by one image per char. 
	 */
	public function createSprite(width:Number, height:Number, text:String, format:starling.text.TextFormat, options:starling.text.TextOptions = undefined):starling.display.Sprite { return null; }
	/**
	 *  Draws text into a QuadBatch. 
	 */
	public function fillMeshBatch(meshBatch:starling.display.MeshBatch, width:Number, height:Number, text:String, format:starling.text.TextFormat, options:starling.text.TextOptions = undefined):void {}
	/**
	 *  @inheritDoc 
	 */
	public function clearMeshBatch(meshBatch:starling.display.MeshBatch):void {}
	/**
	 *  @inheritDoc 
	 */
	public function getDefaultMeshStyle(previousStyle:starling.styles.MeshStyle, format:starling.text.TextFormat, options:starling.text.TextOptions):starling.styles.MeshStyle { return null; }
	/**
	 *  Arranges the characters of text inside a rectangle, adhering to the given settings.
	 *      *  Returns a Vector of BitmapCharLocations.
	 *      *
	 *      *  <p>BEWARE: This method uses an object pool for the returned vector and all
	 *      *  (returned and temporary) BitmapCharLocation instances. Do not save any references and
	 *      *  always call <code>BitmapCharLocation.rechargePool()</code> when you are done processing.
	 *      *  </p>
	 *      
	 */
	public function arrangeChars(width:Number, height:Number, text:String, format:starling.text.TextFormat, options:starling.text.TextOptions):openfl.Vector { return null; }
	/**
	 *  The name of the font as it was parsed from the font file. 
	 */
	public function get name():String { return null; }
	/**
	 *  The native size of the font. 
	 */
	public function get size():Number { return 0; }
	/**
	 *  The type of the bitmap font. @see starling.text.BitmapFontType @default standard 
	 */
	public function get type():String { return null; }
	public function set type(value:String):void {}
	public function get_type():String { return null; }
	public function set_type(value:String):String { return null; }
	/**
	 *  If the font uses a distance field texture, this property returns its spread (i.e.
	 *      *  the width of the blurred edge in points). 
	 */
	public function get distanceFieldSpread():Number { return 0; }
	public function set distanceFieldSpread(value:Number):void {}
	public function get_distanceFieldSpread():Number { return 0; }
	public function set_distanceFieldSpread(value:Number):Number { return 0; }
	/**
	 *  The height of one line in points. 
	 */
	public function get lineHeight():Number { return 0; }
	/**
	 *  The smoothing filter that is used for the texture. 
	 */
	public function get smoothing():String { return null; }
	public function set smoothing(value:String):void {}
	/**
	 *  The baseline of the font. This property does not affect text rendering;
	 *      * it's just an information that may be useful for exact text placement. 
	 */
	public function get baseline():Number { return 0; }
	public function set baseline(value:Number):void {}
	/**
	 *  An offset that moves any generated text along the x-axis (in points).
	 *      * Useful to make up for incorrect font data. @default 0. 
	 */
	public function get offsetX():Number { return 0; }
	public function set offsetX(value:Number):void {}
	/**
	 *  An offset that moves any generated text along the y-axis (in points).
	 *      * Useful to make up for incorrect font data. @default 0. 
	 */
	public function get offsetY():Number { return 0; }
	public function set offsetY(value:Number):void {}
	/**
	 *  The width of a "gutter" around the composed text area, in points.
	 *      *  This can be used to bring the output more in line with standard TrueType rendering:
	 *      *  Flash always draws them with 2 pixels of padding. @default 0.0 
	 */
	public function get padding():Number { return 0; }
	public function set padding(value:Number):void {}
}
}
