package starling.text {
import starling.styles.MeshStyle;
import starling.display.MeshBatch;
/**
 *  A text compositor arranges letters for Starling's TextField. 
 * @externs
 */
public interface ITextCompositor {
	/**
	 *  Draws the given text into a MeshBatch, using the supplied format and options. 
	 */
	function fillMeshBatch(meshBatch:starling.display.MeshBatch, width:Number, height:Number, text:String, format:starling.text.TextFormat, options:starling.text.TextOptions = undefined):void;
	/**
	 *  Clears the MeshBatch (filled by the same class) and disposes any resources that
	 *      *  are no longer needed. 
	 */
	function clearMeshBatch(meshBatch:starling.display.MeshBatch):void;
	/**
	 *  Creates and/or sets up the default MeshStyle to be used for rendering.
	 *      *  If <code>previousStyle</code> has the correct type, it is configured as needed and
	 *      *  then returned; otherwise, a new style is created, configured and returned.
	 *      *  The method may return <code>null</code> if there are no special style requirements. 
	 */
	function getDefaultMeshStyle(previousStyle:starling.styles.MeshStyle, format:starling.text.TextFormat, options:starling.text.TextOptions):starling.styles.MeshStyle;
	/**
	 *  Frees all resources allocated by the compositor. 
	 */
	function dispose():void;
}
}
