import EventDispatcher from "../events/EventDispatcher";
declare namespace starling.text {
	/**
	 *  Dispatched when any property of the instance changes. 
	 */
	export class TextOptions extends EventDispatcher {
		/**
		 *  Creates a new TextOptions instance with the given properties. 
		 */
		constructor(wordWrap?: boolean, autoScale?: boolean);
		/**
		 *  Copies all properties from another TextOptions instance. 
		 */
		copyFrom(options: TextOptions): void;
		/**
		 *  Creates a clone of this instance. 
		 */
		clone(): TextOptions;
		/**
		 *  Indicates if the text should be wrapped at word boundaries if it does not fit into
		 *      *  the TextField otherwise. @default true 
		 */
		get wordWrap(): boolean;
		set wordWrap(value: boolean)
		/**
		 *  Specifies the type of auto-sizing set on the TextField. Custom text compositors may
		 *      *  take this into account, though the basic implementation (done by the TextField itself)
		 *      *  is often sufficient: it passes a very big size to the <code>fillMeshBatch</code>
		 *      *  method and then trims the result to the actually used area. @default none 
		 */
		get autoSize(): string;
		set autoSize(value: string)
		/**
		 *  Indicates whether the font size is automatically reduced if the complete text does
		 *      *  not fit into the TextField. @default false 
		 */
		get autoScale(): boolean;
		set autoScale(value: boolean)
		/**
		 *  Indicates if text should be interpreted as HTML code. For a description
		 *      *  of the supported HTML subset, refer to the classic Flash 'TextField' documentation.
		 *      *  Beware: Only supported for TrueType fonts. @default false 
		 */
		get isHtmlText(): boolean;
		set isHtmlText(value: boolean)
		/**
		 *  The scale factor of any textures that are created during text composition.
		 *      *  @default Starling.contentScaleFactor 
		 */
		get textureScale(): number;
		set textureScale(value: number)
		/**
		 *  The Context3DTextureFormat of any textures that are created during text composition.
		 *      *  @default Context3DTextureFormat.BGRA_PACKED 
		 */
		get textureFormat(): string;
		set textureFormat(value: string)
		/**
		 *  The padding (in points) that's added to the sides of text that's rendered to a Bitmap.
		 *      *  If your text is truncated on the sides (which may happen if the font returns incorrect
		 *      *  bounds), padding can make up for that. Value must be positive. @default 0.0 
		 */
		get padding(): number;
		set padding(value: number)
	}
}
export default starling.text.TextOptions;