import ByteArray from "openfl/utils/ByteArray";
declare namespace starling.textures {
	/**
	 *  A parser for the ATF data format. 
	 */
	export class AtfData {
		/**
		 *  Create a new instance by parsing the given byte array. 
		 */
		constructor(data: ByteArray);
		/**
		 *  Checks the first 3 bytes of the data for the 'ATF' signature. 
		 */
		static isAtfData(data: ByteArray): boolean;
		/**
		 *  The texture format. @see flash.display3D.textures.Context3DTextureFormat 
		 */
		get format(): string;
		/**
		 *  The width of the texture in pixels. 
		 */
		get width(): number;
		/**
		 *  The height of the texture in pixels. 
		 */
		get height(): number;
		/**
		 *  The number of encoded textures. '1' means that there are no mip maps. 
		 */
		get numTextures(): number;
		/**
		 *  Indicates if the ATF data encodes a cube map. Not supported by Starling! 
		 */
		get isCubeMap(): boolean;
		/**
		 *  The actual byte data, including header. 
		 */
		get data(): ByteArray;
	}
}
export default starling.textures.AtfData;