package starling.textures {
/**
 *  The TextureOptions class specifies options for loading textures with the
 *  *  <code>Texture.fromData</code> and <code>Texture.fromTextureBase</code> methods. 
 * @externs
 */
public class TextureOptions {
	/**
	 *  Creates a new instance with the given options. 
	 */
	public function TextureOptions(scale:Number = undefined, mipMapping:Boolean = undefined, format:String = undefined, premultipliedAlpha:Boolean = undefined, forcePotTexture:Boolean = undefined) {}
	/**
	 *  Creates a clone of the TextureOptions object with the exact same properties. 
	 */
	public function clone():starling.textures.TextureOptions { return null; }
	/**
	 *  Copies all properties from another TextureOptions instance. 
	 */
	public function copyFrom(other:starling.textures.TextureOptions):void {}
	/**
	 *  The scale factor, which influences width and height properties. If you pass '-1',
	 *      *  the current global content scale factor will be used. @default 1.0 
	 */
	public function get scale():Number { return 0; }
	public function set scale(value:Number):void {}
	/**
	 *  The <code>Context3DTextureFormat</code> of the underlying texture data. Only used
	 *      *  for textures that are created from Bitmaps; the format of ATF files is set when they
	 *      *  are created. @default BGRA 
	 */
	public function get format():String { return null; }
	public function set format(value:String):void {}
	/**
	 *  Indicates if the texture contains mip maps. @default false 
	 */
	public function get mipMapping():Boolean { return false; }
	public function set mipMapping(value:Boolean):void {}
	/**
	 *  Indicates if the texture will be used as render target. 
	 */
	public function get optimizeForRenderToTexture():Boolean { return false; }
	public function set optimizeForRenderToTexture(value:Boolean):void {}
	/**
	 *  Indicates if the underlying Stage3D texture should be created as the power-of-two based
	 *      *  <code>Texture</code> class instead of the more memory efficient <code>RectangleTexture</code>.
	 *      *  That might be useful when you need to render the texture with wrap mode <code>repeat</code>.
	 *      *  @default false 
	 */
	public function get forcePotTexture():Boolean { return false; }
	public function set forcePotTexture(value:Boolean):void {}
	/**
	 *  If this value is set, the texture will be loaded asynchronously (if possible).
	 *      *  The texture can only be used when the callback has been executed.
	 *      *  
	 *      *  <p>This is the expected function definition: 
	 *      *  <code>function(texture:Texture):void;</code></p>
	 *      *
	 *      *  @default null
	 *      
	 */
	public function get onReady():Function { return null; }
	public function set onReady(value:Function):void {}
	/**
	 *  Indicates if the alpha values are premultiplied into the RGB values. This is typically
	 *      *  true for textures created from BitmapData and false for textures created from ATF data.
	 *      *  This property will only be read by the <code>Texture.fromTextureBase</code> factory
	 *      *  method. @default true 
	 */
	public function get premultipliedAlpha():Boolean { return false; }
	public function set premultipliedAlpha(value:Boolean):void {}
}
}
