package starling.utils {
import openfl.geom.Vector3D;
import openfl.geom.Rectangle;
import openfl.geom.Point;
import openfl.geom.Matrix3D;
import openfl.geom.Matrix;
/**
 *  A simple object pool supporting the most basic utility objects.
 *  *
 *  *  <p>If you want to retrieve an object, but the pool does not contain any more instances,
 *  *  it will silently create a new one.</p>
 *  *
 *  *  <p>It's important that you use the pool in a balanced way, i.e. don't just "get" or "put"
 *  *  alone! Always make the calls in pairs; whenever you get an object, be sure to put it back
 *  *  later, and the other way round. Otherwise, the pool will empty or (even worse) grow
 *  *  in size uncontrolled.</p>
 *  
 * @externs
 */
public class Pool {
	/**
	 *  @private 
	 */
	function Pool() {}
	/**
	 *  Retrieves a Point instance from the pool. 
	 */
	public static function getPoint(x:Number = undefined, y:Number = undefined):openfl.geom.Point { return null; }
	/**
	 *  Stores a Point instance in the pool.
	 *      *  Don't keep any references to the object after moving it to the pool! 
	 */
	public static function putPoint(point:openfl.geom.Point):void {}
	/**
	 *  Retrieves a Vector3D instance from the pool. 
	 */
	public static function getPoint3D(x:Number = undefined, y:Number = undefined, z:Number = undefined):openfl.geom.Vector3D { return null; }
	/**
	 *  Stores a Vector3D instance in the pool.
	 *      *  Don't keep any references to the object after moving it to the pool! 
	 */
	public static function putPoint3D(point:openfl.geom.Vector3D):void {}
	/**
	 *  Retrieves a Matrix instance from the pool. 
	 */
	public static function getMatrix(a:Number = undefined, b:Number = undefined, c:Number = undefined, d:Number = undefined, tx:Number = undefined, ty:Number = undefined):openfl.geom.Matrix { return null; }
	/**
	 *  Stores a Matrix instance in the pool.
	 *      *  Don't keep any references to the object after moving it to the pool! 
	 */
	public static function putMatrix(matrix:openfl.geom.Matrix):void {}
	/**
	 *  Retrieves a Matrix3D instance from the pool.
	 *      *
	 *      *  @param identity   If enabled, the matrix will be reset to the identity.
	 *      *                    Otherwise, its contents is undefined.
	 *      
	 */
	public static function getMatrix3D(identity:Boolean = undefined):openfl.geom.Matrix3D { return null; }
	/**
	 *  Stores a Matrix3D instance in the pool.
	 *      *  Don't keep any references to the object after moving it to the pool! 
	 */
	public static function putMatrix3D(matrix:openfl.geom.Matrix3D):void {}
	/**
	 *  Retrieves a Rectangle instance from the pool. 
	 */
	public static function getRectangle(x:Number = undefined, y:Number = undefined, width:Number = undefined, height:Number = undefined):openfl.geom.Rectangle { return null; }
	/**
	 *  Stores a Rectangle instance in the pool.
	 *      *  Don't keep any references to the object after moving it to the pool! 
	 */
	public static function putRectangle(rectangle:openfl.geom.Rectangle):void {}
}
}
